import plotly.graph_objects as go
import pytest

from commodplot import messaging


@pytest.mark.skip()
def test_compose_and_send_report():
    fig = go.Figure(
        data=[go.Bar(x=[1, 2, 3], y=[1, 3, 2])],
        layout=go.Layout(
            title=go.layout.Title(text="A Figure Specified By A Graph Object")
        ),
    )

    subject = "test_email"
    content = """
        <html>
            <body>
                <h1 style="text-align: center;">Simple Data Report</h1>
                <p>Here could be a short description of the data.</p>
                <p><img src="cid:0"></p>
            </body>
        </html>'
        """

    img = fig.to_image(width=1200, height=350, engine="kaleido")
    images = {"0": img}
    sender = "testcommodplot@mailinator.com"
    messaging.compose_and_send_report(
        subject=subject,
        content=content,
        images=images,
        sender_email=sender,
        receiver_email=sender,
    )


@pytest.mark.skip()
def test_compose_jinja_report():
    fig = go.Figure(
        data=[go.Bar(x=[1, 2, 3], y=[1, 3, 2])],
        layout=go.Layout(
            title=go.layout.Title(text="A Figure Specified By A Graph Object")
        ),
    )

    data = {"name": "test", "fig1": fig}

    subject = "test_email"
    sender = "testcommodplot@mailinator.com"
    messaging.compose_and_send_jinja_report(
        subject=subject,
        data=data,
        template="test_report.html",
        package_loader_name="commodplot",
        sender_email=sender,
        receiver_email=sender,
    )



import pandas as pd
from commodutil import transforms


def seasonalise(df, histfreq):
    """
    Given a dataframe, seasonalise the data, returning seasonalised dataframe
    :param df:
    :return:
    """
    if isinstance(df, pd.Series):
        df = pd.DataFrame(df)

    if histfreq is None:
        histfreq = pd.infer_freq(df.index)
        if histfreq is None:
            histfreq = "D"  # sometimes infer_freq returns null - assume mostly will be a daily series

    if histfreq.startswith("W"):
        seas = transforms.seasonalise_weekly(df, freq=histfreq)
    else:
        seas = transforms.seasonailse(df)

    seas = seas.dropna(how="all", axis=1)  # dont plot empty years
    return seas



import itertools

import cufflinks as cf
import pandas as pd
import plotly as py
import plotly.graph_objects as go
from commodutil import dates
from commodutil import transforms
from plotly.subplots import make_subplots

from commodplot import commodplottrace as cptr
from commodplot import commodplotutil as cpu

preset_margins = {"l": 0, "r": 0, "t": 40, "b": 0}


def seas_line_plot(df, fwd=None, **kwargs):
    """
    Given a DataFrame produce a seasonal line plot (x-axis - Jan-Dec, y-axis Yearly lines)
    Can overlay a forward curve on top of this
    """

    fig = go.Figure()
    traces = cptr.seas_plot_traces(df, fwd, **kwargs)
    if "shaded_range" in traces and traces["shaded_range"]:
        for trace in traces["shaded_range"]:
            fig.add_trace(trace)

    if "average_line" in traces:
        fig.add_trace(traces["average_line"])

    if "hist" in traces:
        for trace in traces["hist"]:
            fig.add_trace(trace)

    if "fwd" in traces:
        for trace in traces["fwd"]:
            fig.add_trace(trace)

    fig.layout.xaxis.tickvals = pd.date_range(
        start=str(dates.curyear), periods=12, freq="MS"
    )

    title = cpu.gen_title(df, **kwargs)
    legend = go.layout.Legend(font=dict(size=10), traceorder="reversed")
    yaxis_title = kwargs.get("yaxis_title", None)
    hovermode = kwargs.get("hovermode", "x")
    fig.update_layout(
        title=title,
        title_x=0.01,
        xaxis_tickformat="%b",
        yaxis_title=yaxis_title,
        legend=legend,
        hovermode=hovermode,
        margin=preset_margins,
    )

    return fig


def seas_line_subplot(rows, cols, df, fwd=None, **kwargs):
    """
    Generate a plot with multiple seasonal subplots.
    :param rows:
    :param cols:
    :param dfs:
    :param fwds:
    :param kwargs:
    :return:
    """
    fig = make_subplots(
        cols=cols,
        rows=rows,
        specs=[[{"type": "scatter"} for x in range(0, cols)] for y in range(0, rows)],
        subplot_titles=kwargs.get("subplot_titles", None),
    )

    chartcount = 0
    for row in range(1, rows + 1):
        for col in range(1, cols + 1):
            # print(row, col)
            if chartcount > len(df):
                chartcount += 1
                continue

            dfx = df[df.columns[chartcount]]
            fwdx = None
            if fwd is not None and len(fwd) > chartcount:
                fwdx = fwd[fwd.columns[chartcount]]

            showlegend = True if chartcount == 0 else False

            traces = cptr.seas_plot_traces(
                dfx, fwd=fwdx, showlegend=showlegend, **kwargs
            )

            for trace_set in ["shaded_range", "hist", "fwd"]:
                if trace_set in traces:
                    for trace in traces[trace_set]:
                        fig.add_trace(trace, row=row, col=col)

            chartcount += 1

    legend = go.layout.Legend(font=dict(size=10))
    fig.update_xaxes(
        tickvals=pd.date_range(start=str(dates.curyear), periods=12, freq="MS"),
        tickformat="%b",
    )
    title = kwargs.get("title", "")
    fig.update_layout(
        title=title,
        title_x=0.01,
        xaxis_tickformat="%b",
        legend=legend,
        margin=preset_margins,
    )
    return fig


def seas_box_plot(hist, fwd=None, **kwargs):
    hist = transforms.monthly_mean(hist)
    hist = hist.T

    data = []
    monthstr = {
        x.month: x.strftime("%b")
        for x in pd.date_range(start="2018", freq="M", periods=12)
    }
    for x in hist.columns:
        trace = go.Box(name=monthstr[x], y=hist[x])
        data.append(trace)

    fwdl = transforms.seasonailse(fwd)
    fwdl.index = fwdl.index.strftime("%b")
    for col in fwdl.columns:
        ser = fwdl[col].copy()
        trace = go.Scatter(
            name=col,
            x=ser.index,
            y=ser,
            line=dict(color=cptr.get_year_line_col(col), dash="dot"),
        )
        data.append(trace)

    fig = go.Figure(data=data)
    title = kwargs.get("title", "")
    fig.update_layout(title=title, title_x=0.01, margin=preset_margins)

    return fig


def seas_table_plot(hist, fwd=None):
    df = cpu.seas_table(hist, fwd)

    colsh = list(df.columns)
    colsh.insert(0, "Period")

    cols = [df[x] for x in df]
    cols.insert(0, list(df.index))
    fillcolor = ["lavender"] * 12
    fillcolor.extend(["aquamarine"] * 4)
    fillcolor.extend(["darkturquoise"] * 2)
    fillcolor.append("dodgerblue")

    figm = go.Figure(
        data=[
            go.Table(
                header=dict(values=colsh, fill_color="paleturquoise", align="left"),
                cells=dict(values=cols, fill_color=[fillcolor], align="left"),
            )
        ]
    )
    return figm


def table_plot(df, **kwargs):
    row_even_colour = kwargs.get("row_even_colour", "lightgrey")
    row_odd_color = kwargs.get("row_odd_colour", "white")

    # include index col as part of plot
    indexname = "" if df.index.name is None else df.index.name
    colheaders = [indexname] + list(df.columns)
    headerfill = ["white" if x == "" else "grey" for x in colheaders]

    cols = [df[x] for x in df.columns]
    # apply red/green to formatted_cols
    fcols = kwargs.get("formatted_cols", [])
    font_color = [
        ["red" if str(y).startswith("-") else "green" for y in df[x]]
        if x in fcols
        else "black"
        for x in colheaders
    ]

    if isinstance(df.index, pd.DatetimeIndex):  # if index is datetime, format dates
        df.index = df.index.map(lambda x: x.strftime("%d-%m-%Y"), 1)
    cols.insert(0, df.index)

    fig = go.Figure(
        data=[
            go.Table(
                header=dict(
                    values=colheaders,
                    fill_color=headerfill,
                    align="center",
                    font=dict(color="white", size=12),
                ),
                cells=dict(
                    values=cols,
                    line=dict(color="#506784"),
                    fill_color=[[row_odd_color, row_even_colour] * len(df)],
                    align="right",
                    font_color=font_color,
                ),
            )
        ]
    )
    return fig


def forward_history_plot(df, title=None, **kwargs):
    """
    Given a dataframe of a curve's pricing history, plot a line chart showing how it has evolved over time
    """
    df = df.rename(columns={x: pd.to_datetime(x) for x in df.columns})
    df = df[sorted(list(df.columns), reverse=True)]  # have latest column first
    df = df.rename(
        columns={x: cpu.format_date_col(x, "%d-%b-%y") for x in df.columns}
    )  # make nice labels for legend eg 05-Dec

    colseq = py.colors.sequential.Aggrnyl
    text = df.index.strftime("%b-%y")

    fig = go.Figure()
    colcount = 0
    for col in df.columns:
        color = colseq[colcount] if colcount < len(colseq) else colseq[-1]
        fig.add_trace(
            go.Scatter(
                x=df.index,
                y=df[col],
                hoverinfo="y",
                name=str(col),
                line=dict(color=color),
                hovertemplate=cptr.hovertemplate_default,
                text=text,
            )
        )

        colcount = colcount + 1

    fig["data"][0]["line"]["width"] = 2.2  # make latest line thicker
    legend = go.layout.Legend(font=dict(size=10))
    yaxis_title = kwargs.get("yaxis_title", None)
    fig.update_layout(
        title=title,
        title_x=0.01,
        xaxis_tickformat="%b-%y",
        yaxis_title=yaxis_title,
        legend=legend,
        margin=preset_margins,
    )
    return fig


def bar_line_plot(df, linecol="Total", **kwargs):
    """
    Give a dataframe, make a stacked bar chart along with overlaying line chart.
    """
    if linecol not in df:
        df[linecol] = df.sum(1, skipna=False)

    barcols = [x for x in df.columns if linecol not in x]
    barspecs = {"kind": "bar", "barmode": "relative", "title": "d", "columns": barcols}
    linespecs = {"kind": "scatter", "columns": linecol, "color": "black"}

    fig = cf.tools.figures(df, [barspecs, linespecs])  # returns dict
    fig = go.Figure(fig)
    yaxis_title = kwargs.get("yaxis_title", None)
    yaxis_range = kwargs.get("yaxis_range", None)
    title = kwargs.get("title", None)
    fig.update_layout(
        title=title,
        title_x=0.01,
        xaxis_title="Date",
        yaxis_title=yaxis_title,
        margin=preset_margins,
    )
    if yaxis_range is not None:
        fig.update_layout(yaxis=dict(range=yaxis_range))
    return fig


def diff_plot(df, **kwargs):
    """
    Given a dataframe, plot each column as line plot with a subplot below
    showing differences between each column.
    :param df:
    :param kwargs:
    :return:
    """
    # calculate difference between each column
    for comb in itertools.combinations(df.columns, 2):
        df["%s-%s" % (comb[0], comb[1])] = df[comb[0]] - df[comb[1]]

    barcols = [x for x in df.columns if "-" in x]
    linecols = [x for x in df.columns if "-" not in x]

    fig = make_subplots(
        rows=2, cols=1, row_heights=[0.8, 0.2], shared_xaxes=True, vertical_spacing=0.02
    )
    for col in linecols:
        fig.add_trace(go.Scatter(x=df.index, y=df[col], name=col))

    for col in barcols:
        fig.add_trace(go.Bar(x=df.index, y=df[col], name=col), row=2, col=1)

    title = kwargs.get("title", "")
    fig.update_layout(title_text=title, title_x=0.01, margin=preset_margins)
    return fig


def reindex_year_line_plot(df, **kwargs):
    """
    Given a dataframe of timeseries, reindex years and produce line plot
    :param df:
    :return:
    """
    fig = go.Figure()
    dft = transforms.reindex_year(df)
    colsel = cpu.reindex_year_df_rel_col(dft)

    traces = cptr.reindex_plot_traces(dft, current_select_year=colsel, **kwargs)

    if "shaded_range" in traces and traces["shaded_range"]:
        for trace in traces["shaded_range"]:
            fig.add_trace(trace)

    if "hist" in traces:
        for trace in traces["hist"]:
            fig.add_trace(trace)

    kwargs["title_postfix"] = colsel
    title = cpu.gen_title(df[colsel], title_prefix=colsel, **kwargs)

    legend = go.layout.Legend(font=dict(size=10))
    yaxis_title = kwargs.get("yaxis_title", None)
    fig.update_layout(
        title=title,
        title_x=0.01,
        xaxis_tickformat="%b-%y",
        yaxis_title=yaxis_title,
        legend=legend,
        margin=preset_margins,
    )
    # zoom into last 3 years
    fig.update_xaxes(
        type="date",
        range=[
            dft.tail(365 * 3).index[0].strftime("%Y-%m-%d"),
            dft.index[-1].strftime("%Y-%m-%d"),
        ],
    )

    return fig


def candle_chart(df, **kwargs):
    fig = go.Figure(
        data=[
            go.Candlestick(
                x=df.index,
                open=df["Open"],
                high=df["High"],
                low=df["Low"],
                close=df["Close"],
            )
        ]
    )

    title = cpu.gen_title(df["Close"], **kwargs)
    fig.update_layout(title=title)
    return fig


def stacked_area_chart(df, **kwargs):
    fig = go.Figure()
    group = kwargs.get("stackgroup", "stackgroup")

    for col in df.columns:
        fig.add_trace(go.Scatter(x=df.index, y=df[col], name=col, stackgroup=group))

    fig.update_layout(title=kwargs.get("title", ""))
    return fig


def bar_chart(df, **kwargs):
    fig = go.Figure()

    for col in df.columns:
        fig.add_trace(go.Bar(x=df.index, y=df[col], name=col))

    hovermode = kwargs.get("hovermode", "x")
    fig.update_layout(title=kwargs.get("title", ""), hovermode=hovermode)
    barmode = kwargs.get("barmode", None)
    if barmode:
        fig.update_layout(barmode=barmode)

    return fig


def reindex_year_line_subplot(rows, cols, dfs, **kwargs):
    fig = make_subplots(
        cols=cols,
        rows=rows,
        specs=[[{"type": "scatter"} for x in range(0, cols)] for y in range(0, rows)],
        subplot_titles=kwargs.get("subplot_titles", None),
        shared_xaxes=False,
    )

    chartcount = 0
    for row in range(1, rows + 1):
        for col in range(1, cols + 1):
            # print(row, col)
            if chartcount > len(dfs):
                chartcount += 1
                continue
            showlegend = True if chartcount == 0 else False

            dfx = dfs[chartcount]
            dft = transforms.reindex_year(dfx)
            colsel = cpu.reindex_year_df_rel_col(dft)
            traces = cptr.reindex_plot_traces(
                dft, current_select_year=colsel, showlegend=showlegend, **kwargs
            )
            for trace_set in ["shaded_range", "hist"]:
                if trace_set in traces:
                    for trace in traces[trace_set]:
                        fig.add_trace(trace, row=row, col=col)

            chartcount += 1

    legend = go.layout.Legend(font=dict(size=10))
    yaxis_title = kwargs.get("yaxis_title", None)
    hovermode = kwargs.get("hovermode", "closest")
    title = kwargs.get("title", "")
    fig.update_layout(
        title=title,
        title_x=0.01,
        xaxis_tickformat="%b-%y",
        yaxis_title=yaxis_title,
        legend=legend,
        hovermode=hovermode,
        margin=preset_margins,
    )

    fig.update_xaxes(type="date")

    return fig


def line_plot(df, fwd=None, **kwargs):
    fig = go.Figure()
    res = cptr.line_plot_traces(df, fwd, **kwargs)
    for trace in res:
        fig.add_trace(trace)

    title = cpu.gen_title(df, inc_change_sum=False, **kwargs)
    legend = go.layout.Legend(font=dict(size=10))
    yaxis_title = kwargs.get("yaxis_title", None)
    hovermode = kwargs.get("hovermode", "closest")
    fig.update_layout(
        title=title,
        title_x=0.01,
        yaxis_title=yaxis_title,
        legend=legend,
        hovermode=hovermode,
    )
    return fig



import io
import logging
import traceback
from email.mime.application import MIMEApplication
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import make_msgid
from os import environ
from pathlib import Path
from smtplib import SMTP, SMTPException
from typing import Union

from commodplot import jinjautils

logger = logging.getLogger(__name__)


class EmailBuilder:
    """Easily compose multipart e-mail messages, set headers and add attachments."""

    def __init__(self):
        self.message = MIMEMultipart()

    def set_sender(self, email: str):
        self.message["From"] = email
        return self

    def set_receiver(self, email: str):
        if isinstance(email, list):
            self.message["To"] = ", ".join(email)
        else:
            self.message["To"] = email
        return self

    def set_bcc(self, email: str):
        self.message["Bcc"] = email
        return self

    def set_subject(self, subject: str):
        self.message["Subject"] = subject
        return self

    def set_body(self, body: str, content_type: str = "html"):
        self.message.attach(MIMEText(body, content_type))
        return self

    def attach_file(
        self, file_name: str, attachment_name: str = None, content_id: str = None
    ):
        file_path = Path(file_name)
        try:
            with file_path.open("rb") as attachment:
                part = MIMEApplication(attachment.read())
        except OSError:
            logger.error(f"Cannot add an e-mail attachment {file_name}")
            raise
        # Add headers describing the attachment.
        # Use either the provided attachment name, or file name.
        content_id = content_id or make_msgid(domain="local")
        content_name = attachment_name or file_path.name
        part.add_header("Content-ID", f"<{content_id}>")
        part.add_header("Content-Disposition", f"attachment; filename= {content_name}")
        # Finally, add attachment to the message.
        self.message.attach(part)
        return self

    def attach_image(self, image: Union[str, bytes], content_id: str = None):
        """Attach image part to the message, optionally set the content-id header."""
        content_id = content_id or make_msgid(domain="energy.local")
        part = MIMEImage(image)
        part.add_header("Content-ID", f"<{content_id}>")
        part.add_header("Content-Disposition", f"inline; filename={content_id}")
        self.message.attach(part)
        return self

    def attached_images(self, images: dict):
        """Attached multiple images to message"""
        for content_id, image in images.items():
            part = MIMEImage(image)
            part.add_header("Content-ID", f"<{content_id}>")
            part.add_header("Content-Disposition", f"inline; filename={content_id}")
            self.message.attach(part)
        return self

    def build(self) -> str:
        """Complete the process and return the entire email message as a text."""
        return self.message.as_string()


def compose_and_send_report(
    subject: str,
    content: str,
    images: dict = None,
    sender_email: str = None,
    receiver_email: str = None,
) -> None:
    """
    Compose an e-mail message containing the report and send.

    Configuration:
    * ENV: SENDER_EMAIL - e mail address of the sender
    * ENV: RECEIVER_EMAIL - email address for the recipients
    * ENV: SMTP_HOST - hostname of the SMTP server
    * ENV: SMTP_PORT - port of the SMTP server (default: 25)
    * ENV: SMTP_TIMEOUT - timeout for SMTP operations (default: 60 seconds)
    """

    if not sender_email:
        sender_email = environ.get("SENDER_EMAIL")
    if not receiver_email:
        receiver_email = environ.get("RECEIVER_EMAIL")
    smtp_host = environ.get("SMTP_HOST")
    smtp_port = int(environ.get("SMTP_PORT", "25"))
    smtp_timeout = int(environ.get("SMTP_TIMEOUT", "60"))
    message = (
        EmailBuilder()
        .set_sender(sender_email)
        .set_receiver(receiver_email)
        .set_subject(subject)
        .set_body(content)
    )
    if images:
        message.attached_images(images=images)

    message = message.build()
    logger.info("Sending report e-mail to %s", receiver_email)
    try:
        with SMTP(smtp_host, smtp_port, timeout=smtp_timeout) as client:
            # client.set_debuglevel(1)
            client.connect(smtp_host, smtp_port)
            client.starttls()
            client.sendmail(sender_email, receiver_email, message)
            client.close()
        logger.info("Report sent successfully")
    except SMTPException as ex:
        logger.exception("Failed to send a report")
        errors = io.StringIO()
        logging.error(traceback.print_exc(file=errors))
        contents = str(errors.getvalue())
        logging.error(contents)
        errors.close()


def compose_and_send_jinja_report(
    subject: str,
    data: dict,
    template: str,
    package_loader_name: str = None,
    sender_email: str = None,
    receiver_email: str = None,
    template_globals=None,
):
    message = jinjautils.render_html(
        data=data,
        template=template,
        package_loader_name=package_loader_name,
        plotly_image_conv_func=jinjautils.convert_dict_plotly_fig_png,
        template_globals=template_globals,
    )
    compose_and_send_report(
        subject=subject,
        content=message,
        sender_email=sender_email,
        receiver_email=receiver_email,
    )



import pandas as pd
import typing as t

table_style = [
    dict(selector="tr:hover", props=[("background", "#D6EEEE")]),
    dict(
        selector="th",
        props=[
            ("color", "#fff"),
            ("border", "1px solid #eee"),
            ("padding", "12px 35px"),
            ("border-collapse", "collapse"),
            ("background", "#1D4477"),
            ("font-size", "18px"),
        ],
    ),
    dict(
        selector="td",
        props=[
            ("border", "1px solid #eee"),
            ("padding", "10px 20px"),
            ("border-collapse", "collapse"),
            ("font-size", "15px"),
        ],
    ),
    dict(
        selector="table",
        props=[
            ("font-family", "Helvetica"),
            ("margin", "25px auto"),
            ("border-collapse", "collapse"),
            ("border", "1px solid #eee"),
            ("border-bottom", "2px solid #00cccc"),
        ],
    ),
    dict(selector="caption", props=[("caption-side", "bottom")]),
    dict(
        selector="tr:nth-child(even)",
        props=[
            ("background-color", "#f2f2f2"),
        ],
    ),
]


def color_accounting(val):
    """
    Takes a scalar and returns a string with
    the css property `'color: red'` for negative
    strings, green otherwise.
    """
    if isinstance(val, (float, int)):
        color = "red" if val < 0 else "green"
    else:
        color = "red" if float(val.replace(",", "")) < 0 else "green"
    return "color: %s" % color


def generate_table(
    df: pd.DataFrame, precision: t.Tuple[int, dict] = None, accounting_col_columns: list = None
):

    if precision:
        if isinstance(precision, int):
            format_var = "{:.%sf}" % precision
            df = df.applymap(lambda x: format_var.format(x))
        elif isinstance(precision, dict):
            for col, col_precision in precision.items():
                format_var = "{:,.%sf}" % col_precision
                df[col] = df[col].map(format_var.format)

    if accounting_col_columns:
        res = df.style.applymap(
            color_accounting, subset=accounting_col_columns
        ).set_table_styles(table_style)
    else:
        res = df.style.set_table_styles(table_style)

    return res.to_html()



import numpy as np
import numpy as np
import pandas as pd
import plotly
import plotly.graph_objects as go
from commodutil import dates
from commodutil import transforms

from commodplot import commodplottransform as cpt
from commodplot import commodplotutil as cpu
from commodplot.commodplotutil import default_line_col, year_col_map

hovertemplate_default = "%{y:.2f}: <i>%{text}</i>"


def get_year_line_col(year):
    """
    Given a year, calculate a consistent line colour across charts
    """
    delta = get_year_line_delta(year)
    return year_col_map.get(delta, default_line_col)


def get_sequence_line_col(seqno: int):
    """
    Given a sequence number select the colour from default plotly palette
    Required for line plot where history and forward should be same color
    :param seqno:
    :return:
    """
    if seqno <= len(plotly.colors.qualitative.Plotly):
        return plotly.colors.qualitative.Plotly[seqno]


def line_visible(year, visible_line_years=None):
    """
    Determine the number of year lines to be visible in seasonal plot
    :param year:
    :param years_to_include:
    :return:
    """
    delta = get_year_line_delta(year)
    if delta is None:
        return None
    if visible_line_years:
        visible_line_years = visible_line_years * -1  # number of years to go back
    else:
        visible_line_years = -5  # default to 5
    # 3 represents number of years in the future to show
    return None if visible_line_years <= delta <= 3 else "legendonly"


def get_year_line_delta(year):
    if isinstance(year, str) and year.isnumeric():
        year = int(year)

    delta = year - dates.curyear
    return delta


def get_year_line_width(year):
    delta = get_year_line_delta(year)
    if delta == 0:
        return 3

    return 2


def clean_seas_df_for_min_max_average(seas, range):
    """
    Given a seasonalised dataframe, clean to handle missing data
    :param seas:
    :return:
    """
    seas = seas.dropna(how="all", axis=1)
    seasf = seas.rename(columns=dates.find_year(seas))

    # only consider when we have full(er) data for a given range
    fulldata = pd.DataFrame(seasf.isna().sum())  # count non-na values
    if (
        not (fulldata == 0).all().iloc[0]
    ):  # line below doesn't apply when we have full data for all columns
        fulldata = fulldata[
            fulldata.apply(lambda x: np.abs(x - x.mean()) / x.std() < 1.5).all(axis=1)
        ]  # filter columns with high emtply values
    seasf = seasf[fulldata.index]  # use these column names only

    if isinstance(range, int):
        end_year = dates.curyear - 1
        start_year = end_year - (range - 1)
    else:
        start_year, end_year = range[0], range[1]

    r = seasf[[x for x in seasf.columns if x >= start_year and x <= end_year]]
    return r


def min_max_mean_range(seas, shaded_range):
    """
    Calculate min and max for seas
    If an int eg 5, then do curyear -1 and curyear -6
    If list then do the years in that list eg 2012-2019
    :param seas:
    :param shaded_range:
    :return:
    """
    r = clean_seas_df_for_min_max_average(seas, shaded_range)

    res = r.copy()
    res["min"] = res.min(1)
    res["max"] = res.max(1)
    res["mean"] = res.mean(1)
    res = res[["min", "max", "mean"]]

    if len(r.columns) >= 2:
        rangeyr = int(len(r.columns))  # end_year - start_year
    else:
        rangeyr = None
    return res, rangeyr


def shaded_range_traces(seas, shaded_range, showlegend=True):
    """
    Given a dataframe, calculate the min/max for every day of the year
    and return this as a trace for the min/max shaded area
    :param seas:
    :param shaded_range:
    :param showlegend:
    :return:
    """
    r, rangeyr = min_max_mean_range(seas, shaded_range)
    if isinstance(shaded_range, int):
        name = "%syr" % rangeyr
    else:
        name = "%s-%s" % (str(shaded_range[0])[-2:], str(shaded_range[1])[-2:])

    if rangeyr is not None:
        traces = []
        max_trace = go.Scatter(
            x=r.index,
            y=r["max"].values,
            fill=None,
            name="%s Max" % name,
            mode="lines",
            line_color="lightsteelblue",
            line_width=0.1,
            showlegend=showlegend,
            legendgroup="min",
        )
        traces.append(max_trace)
        min_trace = go.Scatter(
            x=r.index,
            y=r["min"].values,
            fill="tonexty",
            name="%s Min" % name,
            mode="lines",
            line_color="lightsteelblue",
            line_width=0.1,
            showlegend=showlegend,
            legendgroup="max",
        )
        traces.append(min_trace)
        return traces


def average_line_trace(seas, average_line):
    """
    Given a dataframe, calculate the mean for every day of the year
    and return this as a trace for the average line
    :param seas:
    :param average_line:
    :return:
    """
    r, rangeyr = min_max_mean_range(seas, average_line)
    trace = go.Scatter(
        x=r.index,
        y=r["mean"].values,
        fill=None,
        name="%syr Avg" % rangeyr,
        mode="lines",
        line_width=0.4,
        line_color="darkslategray",
        line=dict(dash="dash"),
        showlegend=True,
        legendgroup="avg",
    )
    return trace


def timeseries_to_seas_trace(
    seas, text, dash=None, showlegend=True, visible_line_years=None
):
    """
    Given a dataframe of reindexed data, generate traces for every year
    :param seas:
    :param text:
    :param dash:
    :param showlegend:
    :return:
    """
    traces = []
    for col in seas.columns:
        trace = go.Scatter(
            x=seas.index,
            y=seas[col],
            hoverinfo="y",
            name=str(col),
            hovertemplate=hovertemplate_default,
            text=text,
            visible=line_visible(col, visible_line_years),
            line=dict(
                color=get_year_line_col(col), dash=dash, width=get_year_line_width(col)
            ),
            showlegend=showlegend,
            legendgroup=str(col),
        )
        traces.append(trace)

    return traces


def timeseries_to_reindex_year_trace(
    dft, text, dash=None, current_select_year=None, showlegend=True
):
    traces = []
    colyearmap = cpu.dates.find_year(dft)

    for col in dft.columns:
        colyear = colyearmap[col]
        width = 1.2
        if current_select_year:  # for current year+ makes lines bolder
            if isinstance(current_select_year, str):
                current_select_year = colyearmap[current_select_year]
            if colyear >= current_select_year:
                width = 2.2
        trace = go.Scatter(
            x=dft.index,
            y=dft[col],
            hoverinfo="y",
            name=str(col),
            hovertemplate=hovertemplate_default,
            text=text,
            visible=line_visible(colyear),
            line=dict(color=get_year_line_col(colyear), dash=dash, width=width),
            showlegend=showlegend,
            legendgroup=str(col),
        )
        traces.append(trace)

    return traces


def seas_plot_traces(df, fwd=None, **kwargs):
    """
    Generate traces for a timeseries that is being turned into a seasonal plot.
    Gererate yearlines for both historical and forward (if provided) and the shaded range
    :param df:
    :param fwd:
    :param kwargs:
    :return:
    """
    res = {}
    histfreq = kwargs.get("histfreq", None)
    if histfreq is None:
        histfreq = cpu.infer_freq(df)
    seas = cpt.seasonalise(df, histfreq=histfreq)

    text = seas.index.strftime("%b")
    if histfreq in ["B", "D", "W"]:
        text = seas.index.strftime("%d-%b")

    showlegend = kwargs.get("showlegend", None)
    visible_line_years = kwargs.get("visible_line_years", None)

    # shaded range
    shaded_range = kwargs.get("shaded_range", None)
    if shaded_range is not None:
        res["shaded_range"] = shaded_range_traces(
            seas, shaded_range, showlegend=showlegend
        )

    # average line
    average_line = kwargs.get("average_line", None)
    if average_line is not None:
        res["average_line"] = average_line_trace(seas, average_line)

    # historical / solid lines
    res["hist"] = timeseries_to_seas_trace(
        seas, text, showlegend=showlegend, visible_line_years=visible_line_years
    )

    # fwd / dotted lines
    if fwd is not None:
        fwdfreq = pd.infer_freq(fwd.index)
        # for charts which are daily, resample the forward curve into a daily series
        if histfreq in ["B", "D"] and fwdfreq in ["MS", "ME"]:
            fwd = transforms.format_fwd(
                fwd, df.index[-1]
            )  # only applies for forward curves
        fwdseas = cpt.seasonalise(fwd, histfreq=fwdfreq)

        res["fwd"] = timeseries_to_seas_trace(
            fwdseas, text, showlegend=showlegend, dash="dot"
        )

    return res


def reindex_plot_traces(df, **kwargs):
    """
    Generate traces for a timeseries that is being turned into a reindex year plot.
    Gererate yearlines for both historical and the shaded range
    :param df:
    :param kwargs:
    :return:
    """
    res = {}
    showlegend = kwargs.get("showlegend", None)
    current_select_year = kwargs.get("current_select_year", None)

    text = df.index.strftime("%d-%b")

    shaded_range = kwargs.get("shaded_range", None)
    if shaded_range is not None:
        res["shaded_range"] = shaded_range_traces(
            df, shaded_range, showlegend=showlegend
        )

    # historical / solid lines
    res["hist"] = timeseries_to_reindex_year_trace(
        df, text, current_select_year=current_select_year, showlegend=showlegend
    )

    return res


def timeseries_trace(series: pd.Series, **kwargs) -> go.Scatter:
    """
    Return a standard timeseries trace for use in a plotly figure
    :param series: Pandas timeseries of data
    :param kwargs: kwargs for various formatting options
    :return:
    """
    series = series.dropna()

    # name
    name = series.name
    if not isinstance(name, str):
        name = str(name)

    # hover text formatting
    hover_date_format = kwargs.get("hover_date_format", "%d-%b-%y")

    t = go.Scatter(
        x=series.index,
        y=series.values,
        hoverinfo="y",
        name=name,
        hovertemplate=kwargs.get("hovertemplate", hovertemplate_default),
        text=series.index.strftime(hover_date_format),
        visible=kwargs.get("visible"),
        line=dict(
            width=kwargs.get("width"),
            color=kwargs.get("color"),
            dash=kwargs.get("dash"),
        ),
        legendgroup=str(kwargs.get("legendgroup")),
        showlegend=kwargs.get("showlegend"),
    )
    return t


def timeseries_trace_by_year(
    series: pd.Series, colyear: int, promptyear: int = None, **kwargs
) -> go.Scatter:
    """
    Return a timeseries trace with formatting applied for a given year
    Use the standardised colors for relative years. Eg current year = black
    Make current and future year lines thicker
    :param series: Pandas timeseries of data
    :param colyear: The year represented by this timeseries (if label is different)
    :param promptyear: Year to use when calculating forward years, eg if end of 2020, then make 2021 the prompt year
    :param kwargs:
    :return:
    """
    width = None
    if promptyear:  # for current year+ makes lines bolder
        if colyear >= promptyear:
            width = 2.2

    visible = line_visible(colyear, visible_line_years=kwargs.get("visible_line_years"))
    color = get_year_line_col(colyear)

    t = timeseries_trace(
        series,
        colyear=colyear,
        promptyear=promptyear,
        width=width,
        visible=visible,
        color=color,
        legendgroup=kwargs.get("legendgroup"),
        showlegend=kwargs.get("showlegend"),
    )
    return t


def line_plot_traces(df, fwd=None, **kwargs):
    """
    Generate traces for a timeseries. If historic and forward values are passed,
    then show solid lines followed by forward lines
    :param df:
    :param kwargs:
    :return:
    """
    traces = []
    colyearmap = cpu.dates.find_year(df)
    colcount = 0
    for col in df.columns:
        colyear = colyearmap[col]
        if isinstance(colyear, int) or (
            isinstance(colyear, str) and colyear.isnumeric()
        ):
            trace = timeseries_trace_by_year(
                df[col], colyear, legendgroup=col
            )  # , text, **kwargs)
        else:
            trace = timeseries_trace(
                df[col], legendgroup=col, color=get_sequence_line_col(colcount)
            )  #

        traces.append(trace)

        if fwd is not None and col in fwd.columns:
            f = fwd[col]
            fwdfreq = pd.infer_freq(f.index)
            if fwdfreq in ["MS", "ME"]:
                f = transforms.format_fwd(
                    f, df.index[-1]
                )  # only applies for forward curves
            if isinstance(colyear, int) or (
                isinstance(colyear, str) and colyear.isnumeric()
            ):
                trace = timeseries_trace_by_year(
                    f,
                    colyear,
                    legendgroup=col,
                    showlegend=False,
                )  # , text, **kwargs)
            else:
                trace = timeseries_trace(
                    f,
                    dash="dash",
                    legendgroup=col,
                    showlegend=False,
                    color=get_sequence_line_col(colcount),
                )
            traces.append(trace)

        colcount = colcount + 1

    return traces



import unittest

import cufflinks as cf
import pandas as pd

from commodplot import commodplotutil as cpu


class TestCommodPlotUtil(unittest.TestCase):
    def test_delta_summary_str(self):
        df = cf.datagen.lines(2, 1000)
        col = df.columns[0]

        m1 = df.iloc[-1, 0]
        m2 = df.iloc[-2, 0]
        diff = m1 - m2
        res = cpu.delta_summary_str(df)

        self.assertIn(str(m1.round(2)), res)
        self.assertIn(str(diff.round(2)), res)

    def test_gen_title(self):
        df = pd.DataFrame([1, 2, 3], columns=["Test"])

        res = cpu.gen_title(df, title=None)
        self.assertTrue(res.startswith("3"))

        res = cpu.gen_title(df, title="TTitle")
        self.assertTrue(res.startswith("TTitle"))
        self.assertTrue(res.endswith("+1"))

        res = cpu.gen_title(df, title="TTitle", title_postfix="post")
        self.assertTrue(res.startswith("TTitle  post:"))
        self.assertTrue(res.endswith("+1"))


if __name__ == "__main__":
    unittest.main()



import os
import unittest

import pandas as pd
import plotly.graph_objects as go
from commodutil import forwards

from commodplot import commodplot


class TestCommodplot(unittest.TestCase):
    def test_seas_line_plot(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        cl = pd.read_csv(
            os.path.join(dirname, "test_cl.csv"),
            index_col=0,
            parse_dates=True,
            dayfirst=True,
        )
        cl = cl.dropna(how="all", axis=1)
        fwd = pd.DataFrame(
            [50 for x in range(12)],
            index=pd.date_range("2021-01-01", periods=12, freq="MS"),
        )

        res = commodplot.seas_line_plot(
            cl[cl.columns[-1]],
            fwd=fwd,
            shaded_range=5,
            visible_line_years=3,
            average_line=5,
        )
        self.assertTrue(isinstance(res, go.Figure))

        shaded_range_max = [x for x in res.data if "Max" in x["name"]]
        self.assertEqual(len(shaded_range_max), 1)
        shaded_range_min = [x for x in res.data if "Min" in x["name"]]
        self.assertEqual(len(shaded_range_min), 1)

        solid_line = [x for x in res.data if x["name"] == "2020"]
        self.assertEqual(solid_line[0].hoverinfo, "y")

        dot_line = [x for x in res.data if x["name"] == "2021"]
        self.assertEqual(dot_line[0].line.dash, "dot")

    def test_seas_line_subplot(self):
        dr = pd.date_range(start="2015", end="2020-12-31", freq="B")
        data = {
            "A": [10 for x in dr],
            "B": [20 for x in dr],
            "C": [30 for x in dr],
            "D": [10 for x in dr],
        }
        df = pd.DataFrame(data, index=dr)
        dr = pd.date_range("2021-01-01", periods=12, freq="MS")
        data = {
            "A": [10 for x in dr],
            "B": [20 for x in dr],
            "C": [30 for x in dr],
            "D": [10 for x in dr],
        }
        fwd = pd.DataFrame(data, index=dr)

        res = commodplot.seas_line_subplot(
            2, 2, df, fwd=fwd, subplot_titles=["1", "2", "3", "4"], shaded_range=5
        )
        self.assertTrue(isinstance(res, go.Figure))
        self.assertEqual(4, [x.name for x in res.data].count("2020"))

    def test_reindex_year_line_plot(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        cl = pd.read_csv(
            os.path.join(dirname, "test_cl.csv"),
            index_col=0,
            parse_dates=True,
            dayfirst=True,
        )
        cl = cl.dropna(how="all", axis=1)
        cl = cl.rename(
            columns={
                x: pd.to_datetime(forwards.convert_contract_to_date(x))
                for x in cl.columns
            }
        )

        sp = forwards.time_spreads(cl, 12, 12)

        res = commodplot.reindex_year_line_plot(sp)
        self.assertTrue(isinstance(res, go.Figure))

    def test_fwd_hist_plot(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        cl = pd.read_csv(
            os.path.join(dirname, "test_cl_fwd.csv"),
            index_col=0,
            parse_dates=True,
            dayfirst=True,
        )
        res = commodplot.forward_history_plot(cl)
        self.assertTrue(isinstance(res, go.Figure))

    def test_candle_chart(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        cl = pd.read_csv(
            os.path.join(dirname, "test_cl_chlo.csv"),
            index_col=0,
            parse_dates=True,
            dayfirst=True,
        )

        res = commodplot.candle_chart(cl)
        self.assertTrue(isinstance(res, go.Figure))

    def test_stack_area_chart(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        cl = pd.read_csv(
            os.path.join(dirname, "test_cl_chlo.csv"),
            index_col=0,
            parse_dates=True,
            dayfirst=True,
        )

        res = commodplot.stacked_area_chart(cl)
        self.assertTrue(isinstance(res, go.Figure))

    def test_stack_bar_chart(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        cl = pd.read_csv(
            os.path.join(dirname, "test_cl_chlo.csv"),
            index_col=0,
            parse_dates=True,
            dayfirst=True,
        )

        res = commodplot.bar_chart(cl, barmode="stack")
        self.assertTrue(isinstance(res, go.Figure))

    def test_reindex_year_line_subplot(self):
        dr = pd.date_range(start="2015", end="2020-12-31", freq="B")
        data = {
            "Q1 2019": [10 for x in dr],
            2020: [20 for x in dr],
            2021: [30 for x in dr],
            2022: [10 for x in dr],
        }
        df = pd.DataFrame(data, index=dr)
        dfs = [df for x in range(1, 5)]

        res = commodplot.reindex_year_line_subplot(
            2, 2, dfs, subplot_titles=["1", "2", "3", "4"]
        )
        self.assertTrue(isinstance(res, go.Figure))

    def test_seas_box_plot(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        cl = pd.read_csv(
            os.path.join(dirname, "test_cl.csv"),
            index_col=0,
            parse_dates=True,
            dayfirst=True,
        )
        cl = cl.dropna(how="all", axis=1)
        fwd = cl[cl.columns[-1]].resample("MS").mean()

        res = commodplot.seas_box_plot(cl[cl.columns[-1]], fwd)
        self.assertTrue(isinstance(res, go.Figure))

    def test_table_plot(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        cl = pd.read_csv(
            os.path.join(dirname, "test_cl.csv"),
            index_col=0,
            parse_dates=True,
            dayfirst=True,
        )
        cl = cl.dropna(how="all", axis=1)

        res = commodplot.table_plot(cl, formatted_cols=["CL_2020F"])
        self.assertTrue(isinstance(res, go.Figure))

    def test_seas_table(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        cl = pd.read_csv(
            os.path.join(dirname, "test_cl.csv"),
            index_col=0,
            parse_dates=True,
            dayfirst=True,
        )
        cl = cl.dropna(how="all", axis=1)
        fwd = cl[cl.columns[-1]].resample("MS").mean()

        res = commodplot.seas_table_plot(cl[cl.columns[-1]], fwd)
        self.assertTrue(isinstance(res, go.Figure))

    def test_diff_plot(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        cl = pd.read_csv(
            os.path.join(dirname, "test_cl.csv"),
            index_col=0,
            parse_dates=True,
            dayfirst=True,
        )
        cl = cl.dropna(how="all", axis=1)[["CL_2020F", "CL_2020G"]]

        res = commodplot.diff_plot(cl, title="Test")
        self.assertTrue(isinstance(res, go.Figure))

    def test_line_plot(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        cl = pd.read_csv(
            os.path.join(dirname, "test_cl.csv"),
            index_col=0,
            parse_dates=True,
            dayfirst=True,
        )
        cl = cl.dropna(how="all", axis=1)[["CL_2019F", "CL_2020G"]]
        fwd = pd.DataFrame(
            [[50 for x in range(2)]],
            index=pd.date_range("2021-01-01", periods=12, freq="MS"),
            columns=["CL_2019F", "CL_2020G"],
        )

        res = commodplot.line_plot(cl, fwd=fwd, title="Test")
        self.assertTrue(isinstance(res, go.Figure))

    def test_line_plot2(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        cl = pd.read_csv(
            os.path.join(dirname, "test_cl.csv"),
            index_col=0,
            parse_dates=True,
            dayfirst=True,
        )
        cl = cl.dropna(how="all", axis=1)[["CL_2020F", "CL_2020G"]]
        cl = cl.rename(columns={"CL_2020F": "A", "CL_2020G": "B"})
        fwd = pd.DataFrame(
            [[50 for x in range(2)]],
            index=pd.date_range("2021-01-01", periods=12, freq="MS"),
            columns=["A", "B"],
        )

        res = commodplot.line_plot(cl, fwd=fwd, title="Test")
        self.assertTrue(isinstance(res, go.Figure))


if __name__ == "__main__":
    unittest.main()



import pandas as pd
import numpy as np
from commodutil import dates
from commodutil import transforms

default_line_col = "khaki"

# try to put deeper colours for recent years, lighter colours for older years
year_col_map = {
    -10: "wheat",
    -9: "burlywood",
    -8: "steelblue",
    -7: "aquamarine",
    -6: "orange",
    -5: "yellow",
    -4: "saddlebrown",
    -3: "mediumblue",
    -2: "darkgreen",
    -1: "coral",
    0: "black",
    1: "red",
    2: "firebrick",
    3: "darkred",
    4: "crimson",
}


def gen_title(df, **kwargs):
    title = kwargs.get("title", "")
    title_postfix = kwargs.get("title_postfix", "")
    inc_change_sum = kwargs.get("inc_change_sum", True)
    precision_format = kwargs.get("precision_format", "")
    if inc_change_sum:
        if title:
            if title_postfix:
                title = "{}  {}: {}".format(title, title_postfix, delta_summary_str(df,precision_format))
            else:
                title = "{}   {}".format(title, delta_summary_str(df,precision_format))
        else:
            if title_postfix:
                title = "{}   {}".format(title_postfix, delta_summary_str(df,precision_format))
            else:
                title = delta_summary_str(df,precision_format)
    else:
        if title:
            if title_postfix:
                title = "{}  {}".format(title, title_postfix)
            else:
                title = title

    return title


def seas_table(hist, fwd=None):
    hist = hist.resample("MS").mean()

    if fwd is not None and fwd.index[0] == hist.index[-1]:
        hist = hist[:-1]
        df = pd.concat([hist, fwd], sort=False)
    else:
        df = hist

    df = transforms.seasonailse(df)

    summary = df.resample("Q").mean()
    winter = summary.iloc[[0, 3], :].mean()
    winter.name = "Q1+Q4"
    summer = summary.iloc[[1, 2], :].mean()
    summer.name = "Q2+Q3"
    summary.index = ["Q1", "Q2", "Q3", "Q4"]
    summary = summary.append(winter)
    summary = summary.append(summer)
    cal = df.resample("Y").mean().iloc[0]
    cal.name = "Year"
    summary = summary.append(cal)
    summary = summary.round(2)

    df.index = df.index.strftime("%b")
    df = pd.concat([df, summary], sort=False).round(2)
    return df


def delta_summary_str(df, precision_format:str = None):
    """
    Given a timeseries, produce a string which shows the latest change
    For example if T-1 value is 50 and T-2 is 45, return 50.00  △: +5
    """
    if isinstance(df, pd.DataFrame):
        df = pd.Series(df[df.columns[0]])

    df = df.dropna()
    val1 = df.iloc[-1]
    val2 = df.iloc[-2]
    delta = (val1 - val2).round(2)
    symb = "+" if delta > 0.0 else ""

    if precision_format:
        val1, delta = precision_format.format(val1), precision_format.format(delta)
    else:
        val1 = val1.round(2)

    s = "{}   △: {}{}".format(val1, symb, delta)
    return s


def format_date_col(col, date_format="%d-%b"):
    """
    Format a column heading as a data
    :param col:
    :param date_format:
    :return:
    """
    try:
        if isinstance(col, str):
            col = pd.to_datetime(col).strftime(date_format)
        if isinstance(col, pd.Timestamp):
            col = col.strftime(date_format)
    except Exception:
        pass  # ignore - just return original

    return col


def reindex_year_df_rel_col(df):
    """
    Given a reindexed year dataframe, figure out which column to use for change summary
    Basic algorithm is use current year, unless you are 10 days from end of dataframe
    :param df:
    :return:
    """
    res_col = df.columns[0]

    years = dates.find_year(df)
    last_val_date = df.index[-1]

    colyears = [x for x in df if str(dates.curyear) in str(x)]
    if len(colyears) > 0:
        res_col = colyears[0]
        relyear = pd.to_datetime(
            "{}-01-01".format(years.get(res_col))
        )  # year of this column

        dft = df[colyears].dropna()
        if len(dft) > 0:
            relcol_date = df[res_col].dropna().index[-1]  # last date of this column

            delta = last_val_date - relcol_date
            if delta.days < 10:
                relyear1 = (relyear + pd.DateOffset(years=1)).year
                relyear1 = [x for x in df.columns if str(relyear1) in str(x)]
                if len(relyear1) > 0:
                    return relyear1[0]
            else:
                return res_col

    return res_col


def infer_freq(df):
    histfreq = (
        "D"  # sometimes infer_freq returns null - assume mostly will be a daily series
    )
    if df is not None:
        histfreq = pd.infer_freq(df.index)

    return histfreq


def std_yr_col(df, asdict=False):
    """
    Given a dataframe with yearly columns, determine the line colour to use
    """

    if isinstance(df, pd.Series):
        df = pd.DataFrame(df)

    yearmap = dates.find_year(df, use_delta=True)
    colmap = {}
    for colname, delta in yearmap.items():
        colmap[colname] = year_col_map.get(delta, default_line_col)

    if asdict:
        return colmap

    # return array of colours to use - this can be passed into cufflift iplot method
    return [colmap[x] for x in df]



import unittest

import pandas as pd

from commodplot import commodplottable as cpt


class TestCommodplotTable(unittest.TestCase):
    def test_generate_table(self):
        df = pd.DataFrame(
            [[1, 2, 3, 4], [5, 6, 7, 8], [3, -5, 6, 7]],
            columns=["Foo", "Bar", "Buzz", "Fuzz"],
            index=["First", "Second", "Third"],
        )
        res = cpt.generate_table(df, accounting_col_columns=["Bar"])
        self.assertIn('<style type="text/css">', res)


if __name__ == "__main__":
    unittest.main()



import unittest

import cufflinks as cf
import pandas as pd
import plotly.graph_objects as go
from commodutil import transforms

from commodplot import commodplottrace as cptr


class TestCommodPlotTrace(unittest.TestCase):
    def test_min_max_range(self):
        df = cf.datagen.lines(1, 5000)
        dft = transforms.seasonailse(df)
        res = cptr.min_max_mean_range(dft, shaded_range=5)
        self.assertTrue(isinstance(res[0], pd.DataFrame))
        self.assertTrue(isinstance(res[1], int))

    def test_timeseries_trace(self):
        df = cf.datagen.lines(1, 5000)
        t = cptr.timeseries_trace(df[df.columns[0]])
        self.assertTrue(isinstance(t, go.Scatter))
        self.assertEqual(t.name, df.columns[0])
        self.assertEqual(t.hovertemplate, cptr.hovertemplate_default)

    def test_timeseries_trace_by_year(self):
        df = cf.datagen.lines(1, 5000)
        df = transforms.seasonailse(df)
        colyear = df.columns[-1]
        t = cptr.timeseries_trace_by_year(df[df.columns[-1]], colyear=colyear)
        self.assertTrue(isinstance(t, go.Scatter))
        self.assertEqual(t.name, str(df.columns[-1]))
        self.assertEqual(
            t.visible, cptr.line_visible(colyear)
        )  # line visible should match results of line_visible()
        self.assertEqual(t.line.color, cptr.get_year_line_col(colyear))


if __name__ == "__main__":
    unittest.main()



import base64
import logging
import os
from datetime import datetime

import plotly as pl
from jinja2 import PackageLoader, FileSystemLoader, Environment
from plotly import graph_objects as go

# margin to use in HTML charts - make charts bigger but leave space for title
narrow_margin = {"l": 2, "r": 2, "t": 30, "b": 10}


def convert_dict_plotly_fig_png(d):
    """
    Given a dict (that might be passed to jinja), convert all plotly figures png
    """
    for k, v in d.items():
        if isinstance(d[k], go.Figure):
            d[k] = plpng(d[k])
        if isinstance(d[k], dict):
            convert_dict_plotly_fig_png(d[k])

    return d


def plpng(fig):
    """
    Given a plotly figure, return it as a png
    """
    image = base64.b64encode(pl.io.to_image(fig)).decode("ascii")
    res = f'<img src="data:image/png;base64,{image}">'
    return res


def convert_dict_plotly_fig_html_div(d):
    """
    Given a dict (that might be passed to jinja), convert all plotly figures of html divs
    """
    for k, v in d.items():
        if isinstance(d[k], go.Figure):
            d[k] = plhtml(d[k])
        if isinstance(d[k], dict):
            convert_dict_plotly_fig_html_div(d[k])

    return d


def plhtml(fig, margin=narrow_margin, **kwargs):
    """
    Given a plotly figure, return it as a div
    """
    if fig is not None:
        fig.update_layout(margin=margin)

        fig.update_xaxes(automargin=True)
        fig.update_yaxes(automargin=True)
        return pl.offline.plot(fig, include_plotlyjs=False, output_type="div")

    return ""


def render_html(
    data,
    template,
    package_loader_name=None,
    template_globals=None,
    plotly_image_conv_func=convert_dict_plotly_fig_html_div,
    filename: str = None,
):
    """
    Using a Jinja2 template, render html file and return as string
    :param data: dict of jinja parameters to include in rendered html
    :param template: absolute location of template file
    :param package_loader_name: if using PackageLoader instead of FileLoader specify package name
    :return:
    """
    data = plotly_image_conv_func(data)

    tdirname, tfilename = os.path.split(os.path.abspath(template))
    if package_loader_name:
        loader = PackageLoader(package_loader_name, "templates")
    else:
        loader = FileSystemLoader(tdirname)
    env = Environment(loader=loader)
    env.finalize = jinja_finalize
    template = env.get_template(tfilename)
    if template_globals:
        for template_global in template_globals:
            template.globals[template_global] = template_globals[template_global]

    output = template.render(
        pagetitle=data["name"], last_gen_time=datetime.now(), data=data
    )

    if filename:
        render_html_to_file(filename, output)

    return output


def render_html_to_file(filename: str, output: str):
    """
    Using a Jinja2 template, render a html file and save to disk
    :param data: dict of jinja parameters to include in rendered html
    :param template: absolute location of template file
    :param filename: location of where rendered html file should be output
    :param package_loader_name: if using PackageLoader instead of FileLoader specify package name
    :return:
    """
    logging.info("Writing html to {}".format(filename))

    with open(filename, "w", encoding="utf8") as fh:
        fh.write(output)

    return filename


def jinja_finalize(value):
    """
    Finalize for jinja which makes empty entries show as blank rather than none
    and converts plotly charts to html divs
    :param value:
    :return:
    """
    if value is None:
        return ""
    if isinstance(value, go.Figure):
        return plhtml(value)
    return value



import os
import unittest

import plotly.express as px
import plotly.graph_objects as go

from commodplot import jinjautils


class TestCommodPlotUtil(unittest.TestCase):
    def test_convert_dict_plotly_fig_html_div(self):
        df = px.data.gapminder().query("country=='Canada'")
        fig = px.line(df, x="year", y="lifeExp", title="Life expectancy in Canada")

        data = {}
        data["ch1"] = fig
        data["el"] = 1
        data["innerd"] = {}
        data["innerd"]["ch2"] = fig

        res = jinjautils.convert_dict_plotly_fig_html_div(data)
        self.assertTrue(isinstance(res["ch1"], str))
        self.assertTrue(isinstance(res["innerd"]["ch2"], str))

    def test_render_html_to_file(self):
        dirname, filename = os.path.split(os.path.abspath(__file__))
        test_out_loc = os.path.join(dirname, "test.html")
        if os.path.exists(test_out_loc):
            os.remove(test_out_loc)

        fig = go.Figure(
            data=[go.Bar(x=[1, 2, 3], y=[1, 3, 2])],
            layout=go.Layout(
                title=go.layout.Title(text="A Figure Specified By A Graph Object")
            ),
        )

        data = {"name": "test", "fig1": fig}

        jinjautils.render_html(
            data,
            template="test_report.html",
            filename="test.html",
            package_loader_name="commodplot",
        )

        self.assertTrue(test_out_loc)
        if os.path.exists(test_out_loc):
            os.remove(test_out_loc)


if __name__ == "__main__":
    unittest.main()
